@@ -1,6 +1,6 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
@@ -33,10 +33,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040903
-pyqt_version_str = "4.9.3"
+pyqt_version = 0x040a00
+pyqt_version_str = "4.10"
 
-sip_min_version = 0x040d03
+sip_min_version = 0x040e03
 
 qt_version = 0
 qt_edition = ""
@@ -74,9 +74,9 @@ dbuslibs = []
 # dynamic linker won't be able to resolve the symbols.  On other systems we
 # assume we can just run qmake by using its full pathname.
 if sys.platform == 'win32':
-    MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH."
+    MSG_CHECK_QMAKE = "Make sure you have a working Qt qmake on your PATH."
 else:
-    MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH or use the -q argument to explicitly specify a working Qt v4 qmake."
+    MSG_CHECK_QMAKE = "Make sure you have a working Qt qmake on your PATH or use the -q argument to explicitly specify a working Qt qmake."
 
 
 def find_default_qmake():
@@ -177,6 +177,9 @@ def create_optparser():
     g.add_option("-T", "--no-timestamp", action="store_true", default=False,
             dest="no_timestamp", help="suppress timestamps in the header "
             "comments of generated code [default: include timestamps]")
+    g.add_option("--no-deprecated", action="store_false", default=True,
+            dest="with_deprecated", help="disable Qt v4 features deprecated "
+            "in Qt v5 [default: enabled]")
 
     if sys.platform != 'win32':
         if sys.platform.startswith('linux') or sys.platform == 'darwin':
@@ -339,7 +342,9 @@ class ConfigurePyQt4:
             0x040701: "Qt_4_7_0",
             0x040702: "Qt_4_7_1",
             0x040800: "Qt_4_7_2",
-            0x050000: "Qt_4_8_0",
+            0x040803: "Qt_4_8_0",
+            0x040804: "Qt_4_8_3",
+            0x050000: "Qt_4_8_4",
             0x060000: "Qt_5_0_0"
         }
 
@@ -351,39 +356,43 @@ class ConfigurePyQt4:
 
         if opts.mwg_ssl_dir:
             ass_lib_dirs = [os.path.join(opts.mwg_ssl_dir, "lib")]
-            ass_libs = ["ssleay32", "libeay32", "QtCore"]
+            ass_libs = ["ssleay32", "libeay32"]
         else:
             ass_lib_dirs = None
-            ass_libs = ["QtCore"]
+            ass_libs = None
 
         # Note that the order in which we check is important for the
         # consolidated module - a module's dependencies must be checked first.
         pyqt_modules.append("QtCore")
 
         check_module("QtGui", "qwidget.h", "new QWidget()")
-        if qt_version < 0x050000: check_module("QtHelp", "qhelpengine.h", "new QHelpEngine(\"foo\")",
-                extra_libs=["QtCore"])
-        if qt_version < 0x050000: check_module("QtMultimedia", "QAudioDeviceInfo",
+        check_module("QtHelp", "qhelpengine.h", "new QHelpEngine(\"foo\")")
+        check_module("QtMultimedia", "QAudioDeviceInfo",
                 "new QAudioDeviceInfo()")
-        if qt_version < 0x050000: check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
-        if qt_version < 0x050000: check_module("QtDBus", "qdbusconnection.h",
-                "QDBusConnection::systemBus()")
-        if qt_version < 0x050000: check_module("QtDeclarative", "qdeclarativeview.h",
+        check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
+
+        # Qt v4.7 was current when we added support for QtDBus and we didn't
+        # bother properly versioning its API.
+        if qt_version >= 0x040700:
+            check_module("QtDBus", "qdbusconnection.h",
+                    "QDBusConnection::systemBus()")
+
+        check_module("QtDeclarative", "qdeclarativeview.h",
                 "new QDeclarativeView()")
-        if qt_version < 0x050000: check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
-        if qt_version < 0x050000: check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
-        if qt_version < 0x050000: check_module("QtScriptTools", "qscriptenginedebugger.h",
+        check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
+        check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
+        check_module("QtScriptTools", "qscriptenginedebugger.h",
                 "new QScriptEngineDebugger()")
-        if qt_version < 0x050000: check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
+        check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
                 extra_libs=sql_libs)
-        if qt_version < 0x050000: check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
-        if qt_version < 0x050000: check_module("QtTest", "QtTest", "QTest::qSleep(0)")
-        if qt_version < 0x050000: check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
-        if qt_version < 0x050000: check_module("QtXml", "qdom.h", "new QDomDocument()")
-        if qt_version < 0x050000: check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
-        if qt_version < 0x050000: check_module("phonon", "phonon/videowidget.h",
+        check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
+        check_module("QtTest", "QtTest", "QTest::qSleep(0)")
+        check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
+        check_module("QtXml", "qdom.h", "new QDomDocument()")
+        check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
+        check_module("phonon", "phonon/videowidget.h",
                 "new Phonon::VideoWidget()")
-        if qt_version < 0x050000: check_module("QtAssistant", "qassistantclient.h",
+        check_module("QtAssistant", "qassistantclient.h",
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
                 extra_libs=ass_libs)
 
@@ -392,11 +401,10 @@ class ConfigurePyQt4:
         elif sipcfg.universal:
             sipconfig.inform("QtDesigner module disabled with universal binaries.")
         else:
-            if qt_version < 0x050000: check_module("QtDesigner", "QExtensionFactory",
+            check_module("QtDesigner", "QExtensionFactory",
                     "new QExtensionFactory()")
 
-        if qt_version < 0x050000: check_module("QAxContainer", "qaxobject.h", "new QAxObject()",
-                extra_libs=["QAxContainer"])
+        check_module("QAxContainer", "qaxobject.h", "new QAxObject()")
 
         if os.path.isdir(os.path.join(src_dir, "dbus")):
             check_dbus()
@@ -810,6 +818,9 @@ class ConfigurePyQt4:
                 if sipcfg.sip_inc_dir != sipcfg.py_inc_dir:
                     inc_path.insert(0, sipcfg.sip_inc_dir)
 
+                if sipcfg.py_conf_inc_dir != sipcfg.py_inc_dir:
+                    inc_path.insert(0, sipcfg.py_conf_inc_dir)
+
                 if opts.bigqt:
                     api_dir = "../../_qt"
                 else:
@@ -927,6 +938,8 @@ include(%s)
         if "QtXml" in pyqt_modules:
             sipconfig.inform("Creating pylupdate4 Makefile...")
 
+            cxxflags_app = sipcfg.build_macros().get("CXXFLAGS_APP", "")
+
             makefile = sipconfig.ProgramMakefile(
                 configuration=sipcfg,
                 build_file=os.path.join(src_dir, "pylupdate", "pylupdate.sbf"),
@@ -941,13 +954,22 @@ include(%s)
                 deployment_target=sipcfg.deployment_target
             )
 
-            makefile.extra_include_dirs.append(os.path.join(src_dir, "pylupdate"))
+            makefile.extra_include_dirs.append(
+                    os.path.join(src_dir, "pylupdate"))
+
+            if cxxflags_app != "":
+                makefile.extra_cxxflags.append(cxxflags_app)
+
             makefile.generate()
             tool.append("pylupdate")
 
             sipconfig.inform("Creating pyrcc4 Makefile...")
 
             makefile = pyrccMakefile()
+
+            if cxxflags_app != "":
+                makefile.extra_cxxflags.append(cxxflags_app)
+
             makefile.generate()
             tool.append("pyrcc")
         else:
@@ -1095,6 +1117,11 @@ def inform_user():
     sipconfig.inform("These PyQt modules will be built: %s." % ", ".join(pyqt_modules))
     sipconfig.inform("The PyQt Python package will be installed in %s." % opts.pyqtmoddir)
 
+    if opts.with_deprecated:
+        sipconfig.inform("PyQt is being built with deprecated Qt v4 features.")
+    else:
+        sipconfig.inform("PyQt is being built without deprecated Qt v4 features.")
+
     if opts.no_docstrings:
         sipconfig.inform("PyQt is being built without generated docstrings.")
     else:
@@ -1275,7 +1302,7 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     opengl = (mname == "QtOpenGL")
 
     qt = [mname]
-    if mname in ("QtOpenGL", "QtWebKit"):
+    if mname in ("QtAssistant", "QtHelp", "QtOpenGL", "QtWebKit"):
         qt.append("QtCore")
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
@@ -1492,11 +1519,17 @@ def set_sip_flags(pyqt):
 
     pyqt is the configuration instance.
     """
+    sip_flags = []
+
     # If we don't check for signed interpreters, we exclude the 'VendorID'
     # feature
     if not opts.vendorcheck:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append("VendorID")
+        sip_flags.append("-x")
+        sip_flags.append("VendorID")
+
+    if not opts.with_deprecated:
+        sip_flags.append("-x")
+        sip_flags.append("PyQt_Deprecated_5_0")
 
     # Handle the platform tag.
     if sys.platform == 'win32':
@@ -1509,35 +1542,40 @@ def set_sip_flags(pyqt):
     else:
         plattag = "WS_X11"
 
-    qt_sip_flags.append("-t")
-    qt_sip_flags.append(plattag)
+    sip_flags.append("-t")
+    sip_flags.append(plattag)
 
     # Handle the Qt version tag.
     verstag = sipconfig.version_to_sip_tag(qt_version, pyqt.qt_version_tags(), "Qt")
 
     # Handle any feature flags.
     for xf in qt_xfeatures:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append(xf)
+        sip_flags.append("-x")
+        sip_flags.append(xf)
 
     if verstag:
-        qt_sip_flags.append("-t")
-        qt_sip_flags.append(verstag)
+        sip_flags.append("-t")
+        sip_flags.append(verstag)
 
     # Handle the version specific Python features.
     if sipcfg.py_version < 0x020400:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append("Py_DateTime")
+        sip_flags.append("-x")
+        sip_flags.append("Py_DateTime")
 
     if sipcfg.py_version < 0x030000:
-        qt_sip_flags.append("-x")
-        qt_sip_flags.append("Py_v3")
+        sip_flags.append("-x")
+        sip_flags.append("Py_v3")
+
+    qt_sip_flags.extend(sip_flags)
 
     # There is an issue creating QObjects while the GIL is held causing
     # deadlocks in multi-threaded applications.  We don't fully understand this
     # yet so we make sure we avoid the problem by always releasing the GIL.
     qt_sip_flags.append("-g")
 
+    # Return the sip flags that sub-modules need to know about.
+    return " ".join(sip_flags)
+
 
 def needed_qt_libs(mname, qt_libs):
     """Add any additional Qt libraries needed by a module to an existing list.
@@ -1740,6 +1778,10 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
 
+    if qt_version >= 0x050000 and opts.with_deprecated:
+        # The name of this macro is very confusing.
+        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x040900')
+
     makefile.generate()
 
 
@@ -1886,6 +1928,16 @@ def get_build_macros(overrides):
     else:
         fname = os.path.join(qt_datadir, "mkspecs", "default")
 
+        # Qt5 doesn't have the 'default' link.
+        if not os.path.isdir(fname):
+            f = get_command_stdout(opts.qmake + " -query QMAKE_SPEC")
+            fname = f.read().strip()
+            if sys.hexversion >= 0x03000000:
+                fname = fname.decode()
+            f.close()
+
+            fname = os.path.join(qt_datadir, "mkspecs", fname)
+
     fname = os.path.join(fname, "qmake.conf")
 
     if not os.access(fname, os.F_OK):
@@ -1908,7 +1960,14 @@ def get_build_macros(overrides):
     if macros is None:
         return None
 
-    # Make sure we have an entry for MOC (which Qt v5 doesn't provide).
+    # Qt5 doesn't seem to support the specific macros so add them if they are
+    # missing.
+    if macros.get("INCDIR_QT", "") == "":
+        macros["INCDIR_QT"] = qt_incdir
+
+    if macros.get("LIBDIR_QT", "") == "":
+        macros["LIBDIR_QT"] = qt_libdir
+
     if macros.get("MOC", "") == "":
         default_moc = os.path.join(qt_bindir, "moc")
         if sys.platform == 'win32':
@@ -2023,7 +2082,10 @@ SOURCES = %s
 #include <QLibraryInfo>
 #include <QTextStream>
 
-// This seems to be missing from the Qt v5 alpha.
+// These seem to be missing from the Qt v5 beta.
+#if !defined(QT_EDITION_DESKTOP)
+#define QT_EDITION_DESKTOP      8
+#endif
 #if !defined(QT_EDITION_OPENSOURCE)
 #define QT_EDITION_OPENSOURCE   8
 #endif
@@ -2106,6 +2168,10 @@ int main(int argc, char **argv)
     out << "PyQt_PrintPreviewWidget\\n";
 #endif
 
+#if defined(QT_NO_RAWFONT)
+    out << "PyQt_RawFont\\n";
+#endif
+
 #if !defined(QT3_SUPPORT) || QT_VERSION >= 0x040200
     out << "PyQt_NoPrintRangeBug\\n";
 #endif
@@ -2152,7 +2218,7 @@ int main(int argc, char **argv)
     run_command(exe_file)
 
     if not os.access(out_file, os.F_OK):
-        sipconfig.error("%s failed to create %s. Make sure your Qt v4 installation is correct." % (exe_file, out_file))
+        sipconfig.error("%s failed to create %s. Make sure your Qt installation is correct." % (exe_file, out_file))
 
     # Read the directories.
     f = open(out_file, "r")
@@ -2289,11 +2355,23 @@ def main():
     check_vendorid()
 
     # Set the SIP platform, version and feature flags.
-    set_sip_flags(pyqt)
+    sip_flags = set_sip_flags(pyqt)
 
     # Tell the user what's been found.
     inform_user()
 
+    # Embed the sip flags.
+    sipconfig.inform("Embedding sip flags...")
+    in_f = open(os.path.join(src_dir, "qpy", "QtCore", "qpycore_post_init.cpp.in"))
+    out_f = open(os.path.join(src_dir, "qpy", "QtCore", "qpycore_post_init.cpp"), "wt")
+
+    for line in in_f:
+        line = line.replace("@@PYQT_SIP_FLAGS@@", sip_flags)
+        out_f.write(line)
+
+    in_f.close()
+    out_f.close()
+
     # Generate the code.
     pyqt.code()
 
